# TODO: at some point this must be a real Singleton
import pickle


class Creation:
    """
    A creation is a container for all the things built using McThings.
    It can be used to unbuild or build a complete creation. The things
    data will be added during the construction, and it the future it will
    be loaded from a file or other kind of data storage.

    Before adding Things to the Creation, it must be connected to a
    Minecrfat server (fill the Creation.server attribute)
    """

    things = []
    """ map with the things in the creation"""
    server = None
    """ Minecraft server in which create things"""

    @classmethod
    def build(cls):
        """ Recover all the map building the things"""
        for thing in cls.things:
            thing.build()

    @classmethod
    def unbuild(cls):
        """ Recover all the map unbuilding the things """
        for thing in cls.things:
            thing.unbuild()

    @classmethod
    def load(cls, file_path):
        """ Load a creation from a file """
        Creation.things = pickle.load(open(file_path, "rb"))

    @classmethod
    def save(cls, file_path):
        """ Save a creation to a file """
        pickle.dump(Creation.things, open(file_path, "wb"))





import mcpi.block
from mcpi.vec3 import Vec3

from .creation import Creation
from .thing import Thing


class River(Thing):

    length = 100
    width = 2
    depth = 1

    def build(self):

        init_x = self.position.x
        init_y = self.position.y - self.depth
        init_z = self.position.z

        end_x = init_x + self.width - 1
        end_y = self.position.y - 1
        end_z = init_z + self.length - 1

        Creation.server.setBlocks(
            init_x, init_y, init_z,
            end_x, end_y, end_z,
            mcpi.block.WATER_FLOWING)

        self._end_position = Vec3(end_x, end_y + self.depth, end_z)



from math import floor

from mcpi.vec3 import Vec3

from .creation import Creation
from .thing import Thing


class Bridge(Thing):

    large = 5

    def build(self):
        # large = 2 * self.height - 1
        max_height = floor((self.large + 1) / 2)

        for x in range(0, self.large):
            if x < max_height:
                y = x
            elif x == max_height and self.large % 2 == 0:
                y = x - 1
            else:
                y = y - 1
            Creation.server.setBlock(
                self.position.x + x, self.position.y + y, self.position.z,
                self.block)

        self._end_position = Vec3(self.position.x + self.large - 1,
                                  self.position.y,
                                  self.position.z
                                  )
